<?php

namespace WPIS;

require_once 'class-wpis-import-estates.php';

class Cron {

    private $token;
    private $maxImport = 5;
    private $currentImport = 1;
    private $error = false;
    private $action = null;
    private $importSettings = [];
    private $allowedActions = ['import', 'update', 'remove'];
    private $cacheDirectory;

    public function __construct() {

        $this->cacheDirectory = wp_get_upload_dir()['basedir'] . '/wp-immo-sync';

        date_default_timezone_set('Europe/Brussels');
        set_error_handler([$this, 'errorHandler'], E_ALL);

        if( ! is_dir($this->cacheDirectory) ){
            mkdir($this->cacheDirectory);
        }

        $this->logFile = $this->cacheDirectory . '/wpis-cron.log';
        if( !file_exists( $this->logFile ) ){
            touch( $this->logFile );
        }

        $this->wp_db = $GLOBALS['wpdb'];
        $this->wp_table_settings = $this->wp_db->prefix . 'wpis_settings';

        $this->checkAuthorisation();

        if( !$this->error ){
            $this->log('[CRON] ' . strtoupper($this->action) . ' - START');
            switch( $this->action ){
                case 'import':
                    $this->importSettings = (object) ['updateOnly' => false, 'forcePictureImport' => false];
                    $this->import();
                    break;
                case 'update':
                    $this->update();
                    break;
                case 'remove':
                    $this->remove();
                    break;
            }
        }
    }

    public function errorHandler($e, $errstr) {

        if( in_array($e, [E_WARNING, E_ERROR]) ){

            if( file_exists($this->cacheDirectory . '/wpis.lock') ){
                unlink($this->cacheDirectory . '/wpis.lock');
                $this->log('[CRON] ' . strtoupper($this->action) . ' - LOCK AUTO REMOVED DUE TO AN ERROR');
            }

            $this->log('[CRON] ' . strtoupper($this->action) . '  ERROR : ' . json_encode($errstr));

            throw new \Exception($errstr);
        }
    }

    public function checkAuthorisation() {

        $result = $this->wp_db->get_results("SELECT name, value FROM $this->wp_table_settings WHERE name = 'enableCrons'");

        if( !empty($result) && intval($result[0]->value) !== 1 ){
            $this->log('[CRON] CRON ARE DISABLED');
            $this->error = true;
            return $this->response(['success' => false, 'message' => 'Cron are disabled.']);
        }

        $token = $this->wp_db->get_results("SELECT name, value FROM $this->wp_table_settings WHERE name = 'cronKey'");

        if( !isset($_GET['token']) || $_GET['token'] !== $token[0]->value ){
            $this->log('[CRON] INVALID TOKEN ' . (isset($_GET['token']) ? $_GET['token'] : ''));
            $this->error = true;
            return $this->response(['success' => false, 'message' => 'Invalid token.']);
        }

        if( !isset($_GET['action']) || !in_array($_GET['action'], $this->allowedActions) ){
            $this->log('[CRON] INVALID ACTION ' . (isset($_GET['action']) ? $_GET['action'] : ''));
            $this->error = true;
            return $this->response(['success' => false, 'message' => 'Invalid action.']);
        }

        $this->action = $_GET['action'];
    }

    private function import() {

        $importEstates = ImportEstates::init();
        $result = $importEstates->next($this->importSettings);

        if( $result['success'] && ! $result['isComplete'] && $this->currentImport < $this->maxImport ){
            $this->log('[CRON] ' . strtoupper($this->action) . ' - ' . "postId: {$result['lastImport']['postId']}, estateId: {$result['lastImport']['estateId']}, time: {$result['time']} ({$this->currentImport}/{$this->maxImport})");
            $this->currentImport++;
            $this->import();
        }
        else {
            if( !$result['success'] ){
                $this->log('[CRON] ' . strtoupper($this->action) . ' - ' . $result['message']);
            }
            if( $this->action === 'import' && isset($result['lastImport'])){
                $this->log('[CRON] ' . strtoupper($this->action) . ' - ' . "postId: {$result['lastImport']['postId']}, estateId: {$result['lastImport']['estateId']}, time: {$result['time']} ({$this->currentImport}/{$this->maxImport})");
            }
            if( $result['isComplete'] ){
                $importEstates->cache->flush();
                $this->log('[CRON] ' . strtoupper($this->action) . ' - Nothing to import');
            }

            $this->log('[CRON] ' . strtoupper($this->action) . ' - END');
            $this->response(['success' => true]);
        }
    }

    private function update() {
        $importEstates = ImportEstates::init();
        $result = $importEstates->updateStart();
        $importEstates->cache->flush();
        $this->log('[CRON] ' . strtoupper($this->action) . ' - estatesUpdatedCount: ' . $result['estatesUpdatedCount'] . ', time: ' . $result['time']);
        $this->log('[CRON] ' . strtoupper($this->action) . ' - END');
        $this->response(['success' => true]);
    }

    private function remove() {
        $i = 1;
        $importEstates = ImportEstates::init();
        $result = $importEstates->removeInactives();

        if( isset($result['success']) && !$result['success']){
            $this->log('[CRON] ' . strtoupper($this->action) . ' - ERROR : ' . $result['message']);
            $this->log('[CRON] ' . strtoupper($this->action) . ' - END');
            return $this->response($result);
        }

        $total = count($result['estatesRemovedList']);
        foreach( $result['estatesRemovedList'] as $estate){
            $this->log('[CRON] ' . strtoupper($this->action) . " postId: {$estate->postId}, estateId: {$estate->estateId}  ({$i}/{$total})");
            $i++;
        }

        $this->log('[CRON] ' . strtoupper($this->action) . ' - END');

        $importEstates->cache->flush();

        return $this->response(['success' => true]);
    }

    private function log( $message ){
        $date = (new \DateTime('NOW'))->format("y:m:d H:i:s");
        $logLine = "\r\n" . "{$date} " . $message;
        file_put_contents($this->logFile, $logLine, FILE_APPEND);
    }

    private function response( Array $response ) {
        header('Content-Type: application/json');
        echo json_encode( $response );
    }
}

new Cron();